# frozen_string_literal: true

require "active_record"

require "forwardable"

require "yesql/utils/read"
require "yesql/bindings/extractor"
require "yesql/common/adapter"

module ::YeSQL
  class Statement
    extend ::Forwardable

    include ::YeSQL::Common::Adapter
    include ::YeSQL::Utils::Read
    # Give access to the quote method.
    include ::ActiveRecord::ConnectionAdapters::Quoting

    def initialize(bindings = {}, file_path)
      @bindings = bindings
      @file_path = file_path
    end

    def bound
      to_s.gsub(::YeSQL::BIND_REGEX) do |match|
        extract_bind_values(extractor[match[/(\w+)/].to_sym])
      end
    end

    def to_s
      @to_s ||= statement(readable: true)
    end

    def view?
      to_s =~ /^create\s.*view\s/i
    end

    private

    attr_reader :bindings, :file_path

    def extract_bind_values(match)
      return quote(match[:value]) if view?

      match[:bind][:vars]
    end

    def extractor
      ::YeSQL::Bindings::Extractor.new(bindings: bindings).call
    end
  end
end
